@@ -159,34 +159,26 @@
 //!    // Parses addresses (including comments), lists and groups
 //!    assert_eq!(
 //!        message.get_from(),
-//!        &HeaderValue::Address(Addr {
-//!            name: Some("Art Vandelay (Vandelay Industries)".into()),
-//!            address: Some("art@vandelay.com".into())
-//!        })
+//!        &HeaderValue::Address(Addr::new(
+//!            "Art Vandelay (Vandelay Industries)".into(),
+//!            "art@vandelay.com"
+//!        ))
 //!    );
+//!    
 //!    assert_eq!(
 //!        message.get_to(),
 //!        &HeaderValue::GroupList(vec![
-//!            Group {
-//!                name: Some("Colleagues".into()),
-//!                addresses: vec![Addr {
-//!                    name: Some("James Smythe".into()),
-//!                    address: Some("james@vandelay.com".into())
-//!                }]
-//!            },
-//!            Group {
-//!                name: Some("Friends".into()),
-//!                addresses: vec![
-//!                    Addr {
-//!                        name: None,
-//!                        address: Some("jane@example.com".into())
-//!                    },
-//!                    Addr {
-//!                        name: Some("John Smîth".into()),
-//!                        address: Some("john@example.com".into())
-//!                    }
+//!            Group::new(
+//!                "Colleagues",
+//!                vec![Addr::new("James Smythe".into(), "james@vandelay.com")]
+//!            ),
+//!            Group::new(
+//!                "Friends",
+//!                vec![
+//!                    Addr::new(None, "jane@example.com"),
+//!                    Addr::new("John Smîth".into(), "john@example.com"),
 //!                ]
-//!            }
+//!            )
 //!        ])
 //!    );
 //!
@@ -259,10 +251,17 @@
 pub mod decoders;
 pub mod parsers;
 
-use std::{borrow::Cow, collections::HashMap, fmt};
+use std::{
+    borrow::Cow,
+    collections::HashMap,
+    fmt::{self, Display},
+};
 
 use decoders::html::{html_to_text, text_to_html};
-use parsers::fields::thread::thread_name;
+use parsers::{
+    fields::thread::thread_name,
+    preview::{preview_html, preview_text},
+};
 #[cfg(feature = "serde_support")]
 use serde::{Deserialize, Serialize};
 
@@ -273,9 +272,8 @@ pub struct Message<'x> {
     #[cfg_attr(feature = "serde_support", serde(default))]
     pub headers_rfc: RfcHeaders<'x>,
     #[cfg_attr(feature = "serde_support", serde(default))]
-    pub headers_other: OtherHeaders<'x>,
-    #[cfg_attr(feature = "serde_support", serde(default))]
-    pub headers_offsets: Vec<HeaderOffset<'x>>,
+    #[cfg_attr(feature = "serde_support", serde(skip))]
+    pub headers_raw: RawHeaders<'x>,
 
     #[cfg_attr(feature = "serde_support", serde(default))]
     pub html_body: Vec<MessagePartId>,
@@ -313,19 +311,32 @@ impl Default for MessageStructure {
     }
 }
 
-// Part of the message.
+/// Part of the message.
 #[derive(Debug, Default, PartialEq)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
 pub struct Part<'x, T> {
     #[cfg_attr(feature = "serde_support", serde(default))]
-    pub headers: RfcHeaders<'x>,
-
+    pub headers_rfc: RfcHeaders<'x>,
+    #[cfg_attr(feature = "serde_support", serde(default))]
+    #[cfg_attr(feature = "serde_support", serde(skip))]
+    pub headers_raw: RawHeaders<'x>,
+    pub is_encoding_problem: bool,
     pub body: T,
 }
 
 impl<'x, T> Part<'x, T> {
-    pub fn new(headers: RfcHeaders<'x>, body: T) -> Self {
-        Self { headers, body }
+    pub fn new(
+        headers_rfc: RfcHeaders<'x>,
+        headers_raw: RawHeaders<'x>,
+        body: T,
+        is_encoding_problem: bool,
+    ) -> Self {
+        Self {
+            headers_rfc,
+            headers_raw,
+            body,
+            is_encoding_problem,
+        }
     }
 
     pub fn get_body(&self) -> &T {
@@ -333,6 +344,26 @@ impl<'x, T> Part<'x, T> {
     }
 }
 
+impl<'x> MultiPart<'x> {
+    pub fn new(headers_rfc: RfcHeaders<'x>, headers_raw: RawHeaders<'x>) -> Self {
+        Self {
+            headers_rfc,
+            headers_raw,
+        }
+    }
+}
+
+/// Multipart part
+#[derive(Debug, Default, PartialEq)]
+#[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
+pub struct MultiPart<'x> {
+    #[cfg_attr(feature = "serde_support", serde(default))]
+    pub headers_rfc: RfcHeaders<'x>,
+    #[cfg_attr(feature = "serde_support", serde(default))]
+    #[cfg_attr(feature = "serde_support", serde(skip))]
+    pub headers_raw: RawHeaders<'x>,
+}
+
 /// Unique ID representing a MIME part within a message.
 pub type MessagePartId = usize;
 
@@ -364,7 +395,7 @@ pub enum MessagePart<'x> {
     Message(Part<'x, MessageAttachment<'x>>),
 
     /// Multipart part
-    Multipart(RfcHeaders<'x>),
+    Multipart(MultiPart<'x>),
 }
 
 impl<'x> MessagePart<'x> {
@@ -420,6 +451,22 @@ pub struct Addr<'x> {
     pub address: Option<Cow<'x, str>>,
 }
 
+impl<'x> Addr<'x> {
+    pub fn new(name: Option<&'x str>, address: &'x str) -> Self {
+        Self {
+            name: name.map(|name| name.into()),
+            address: Some(address.into()),
+        }
+    }
+
+    pub fn into_owned<'y>(self) -> Addr<'y> {
+        Addr {
+            name: self.name.map(|s| s.into_owned().into()),
+            address: self.address.map(|s| s.into_owned().into()),
+        }
+    }
+}
+
 /// An RFC5322 address group.
 #[derive(Debug, PartialEq)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
@@ -433,31 +480,55 @@ pub struct Group<'x> {
     pub addresses: Vec<Addr<'x>>,
 }
 
-pub type RfcHeaders<'x> = HashMap<HeaderName, HeaderValue<'x>>;
-pub type OtherHeaders<'x> = HashMap<Cow<'x, str>, HeaderValue<'x>>;
+impl<'x> Group<'x> {
+    pub fn new(name: &'x str, addresses: Vec<Addr<'x>>) -> Self {
+        Self {
+            name: Some(name.into()),
+            addresses,
+        }
+    }
+
+    pub fn into_owned<'y>(self) -> Group<'y> {
+        Group {
+            name: self.name.map(|s| s.into_owned().into()),
+            addresses: self.addresses.into_iter().map(|a| a.into_owned()).collect(),
+        }
+    }
+}
+
+pub type RfcHeaders<'x> = HashMap<RfcHeader, HeaderValue<'x>>;
+pub type RawHeaders<'x> = HashMap<HeaderName<'x>, Vec<HeaderOffset>>;
 
 /// Offset of a message element in the raw message.
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Eq, Clone)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
-pub struct HeaderOffset<'x> {
-    pub name: HeaderOffsetName<'x>,
+pub struct HeaderOffset {
     pub start: usize,
     pub end: usize,
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Eq, Hash, Clone)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
-pub enum HeaderOffsetName<'x> {
-    Rfc(HeaderName),
+pub enum HeaderName<'x> {
+    Rfc(RfcHeader),
     Other(Cow<'x, str>),
 }
 
+impl<'x> HeaderName<'x> {
+    pub fn into_owned<'y>(&self) -> HeaderName<'y> {
+        match self {
+            HeaderName::Rfc(header) => HeaderName::Rfc(*header),
+            HeaderName::Other(name) => HeaderName::Other(name.clone().into_owned().into()),
+        }
+    }
+}
+
 /// A header field
 #[repr(u8)]
 #[derive(Debug, PartialEq, Eq, Hash, Clone, Copy)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "serde_support", serde(rename_all = "snake_case"))]
-pub enum HeaderName {
+pub enum RfcHeader {
     Subject = 0,
     From = 1,
     To = 2,
@@ -495,7 +566,50 @@ pub enum HeaderName {
     ListPost = 34,
     ListSubscribe = 35,
     ListUnsubscribe = 36,
-    Other = 37,
+}
+
+impl Display for RfcHeader {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            RfcHeader::Subject => write!(f, "Subject"),
+            RfcHeader::From => write!(f, "From"),
+            RfcHeader::To => write!(f, "To"),
+            RfcHeader::Cc => write!(f, "Cc"),
+            RfcHeader::Date => write!(f, "Date"),
+            RfcHeader::Bcc => write!(f, "Bcc"),
+            RfcHeader::ReplyTo => write!(f, "Reply-To"),
+            RfcHeader::Sender => write!(f, "Sender"),
+            RfcHeader::Comments => write!(f, "Comments"),
+            RfcHeader::InReplyTo => write!(f, "In-Reply-To"),
+            RfcHeader::Keywords => write!(f, "Keywords"),
+            RfcHeader::Received => write!(f, "Received"),
+            RfcHeader::MessageId => write!(f, "Message-ID"),
+            RfcHeader::References => write!(f, "References"),
+            RfcHeader::ReturnPath => write!(f, "Return-Path"),
+            RfcHeader::MimeVersion => write!(f, "MIME-Version"),
+            RfcHeader::ContentDescription => write!(f, "Content-Description"),
+            RfcHeader::ContentId => write!(f, "Content-ID"),
+            RfcHeader::ContentLanguage => write!(f, "Content-Language"),
+            RfcHeader::ContentLocation => write!(f, "Content-Location"),
+            RfcHeader::ContentTransferEncoding => write!(f, "Content-Transfer-Encoding"),
+            RfcHeader::ContentType => write!(f, "Content-Type"),
+            RfcHeader::ContentDisposition => write!(f, "Content-Disposition"),
+            RfcHeader::ResentTo => write!(f, "Resent-To"),
+            RfcHeader::ResentFrom => write!(f, "Resent-From"),
+            RfcHeader::ResentBcc => write!(f, "Resent-Bcc"),
+            RfcHeader::ResentCc => write!(f, "Resent-Cc"),
+            RfcHeader::ResentSender => write!(f, "Resent-Sender"),
+            RfcHeader::ResentDate => write!(f, "Resent-Date"),
+            RfcHeader::ResentMessageId => write!(f, "Resent-Message-ID"),
+            RfcHeader::ListArchive => write!(f, "List-Archive"),
+            RfcHeader::ListHelp => write!(f, "List-Help"),
+            RfcHeader::ListId => write!(f, "List-ID"),
+            RfcHeader::ListOwner => write!(f, "List-Owner"),
+            RfcHeader::ListPost => write!(f, "List-Post"),
+            RfcHeader::ListSubscribe => write!(f, "List-Subscribe"),
+            RfcHeader::ListUnsubscribe => write!(f, "List-Unsubscribe"),
+        }
+    }
 }
 
 /// A parsed header value.
@@ -544,22 +658,22 @@ impl<'x> HeaderValue<'x> {
         *self == HeaderValue::Empty
     }
 
-    pub fn unwrap_text(&mut self) -> Cow<'x, str> {
-        match std::mem::take(self) {
+    pub fn unwrap_text(self) -> Cow<'x, str> {
+        match self {
             HeaderValue::Text(s) => s,
             _ => panic!("HeaderValue::unwrap_text called on non-Text value"),
         }
     }
 
-    pub fn unwrap_datetime(&mut self) -> DateTime {
-        match std::mem::take(self) {
+    pub fn unwrap_datetime(self) -> DateTime {
+        match self {
             HeaderValue::DateTime(d) => d,
             _ => panic!("HeaderValue::unwrap_datetime called on non-DateTime value"),
         }
     }
 
-    pub fn unwrap_content_type(&mut self) -> ContentType<'x> {
-        match std::mem::take(self) {
+    pub fn unwrap_content_type(self) -> ContentType<'x> {
+        match self {
             HeaderValue::ContentType(c) => c,
             _ => panic!("HeaderValue::unwrap_content_type called on non-ContentType value"),
         }
@@ -568,7 +682,7 @@ impl<'x> HeaderValue<'x> {
     pub fn as_text_ref(&self) -> Option<&str> {
         match *self {
             HeaderValue::Text(ref s) => Some(s),
-            HeaderValue::TextList(ref l) => l.get(0)?.as_ref().into(),
+            HeaderValue::TextList(ref l) => l.last()?.as_ref().into(),
             _ => None,
         }
     }
@@ -593,6 +707,40 @@ impl<'x> HeaderValue<'x> {
             _ => None,
         }
     }
+
+    pub fn into_owned<'y>(self) -> HeaderValue<'y> {
+        match self {
+            HeaderValue::Address(addr) => HeaderValue::Address(addr.into_owned()),
+            HeaderValue::AddressList(list) => {
+                HeaderValue::AddressList(list.into_iter().map(|addr| addr.into_owned()).collect())
+            }
+            HeaderValue::Group(group) => HeaderValue::Group(group.into_owned()),
+            HeaderValue::GroupList(list) => {
+                HeaderValue::GroupList(list.into_iter().map(|group| group.into_owned()).collect())
+            }
+            HeaderValue::Text(text) => HeaderValue::Text(text.into_owned().into()),
+            HeaderValue::TextList(list) => HeaderValue::TextList(
+                list.into_iter()
+                    .map(|text| text.into_owned().into())
+                    .collect(),
+            ),
+            HeaderValue::DateTime(datetime) => HeaderValue::DateTime(datetime),
+            HeaderValue::ContentType(ct) => HeaderValue::ContentType(ContentType {
+                c_type: ct.c_type.into_owned().into(),
+                c_subtype: ct.c_subtype.map(|s| s.into_owned().into()),
+                attributes: ct.attributes.map(|attributes| {
+                    attributes
+                        .into_iter()
+                        .map(|(k, v)| (k.into_owned().into(), v.into_owned().into()))
+                        .collect()
+                }),
+            }),
+            HeaderValue::Collection(list) => {
+                HeaderValue::Collection(list.into_iter().map(|v| v.into_owned()).collect())
+            }
+            HeaderValue::Empty => HeaderValue::Empty,
+        }
+    }
 }
 
 /// An RFC2047 Content-Type or RFC2183 Content-Disposition MIME header field.
@@ -623,214 +771,215 @@ pub struct DateTime {
 
 impl<'x> Message<'x> {
     /// Returns an iterator over the RFC headers of this message.
-    pub fn get_headers_rfc(&self) -> impl Iterator<Item = (&HeaderName, &HeaderValue<'x>)> {
+    pub fn get_headers_rfc(&self) -> impl Iterator<Item = (&RfcHeader, &HeaderValue<'x>)> {
         self.headers_rfc.iter()
     }
 
     /// Returns the BCC header field
     pub fn get_bcc(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::Bcc)
+            .get(&RfcHeader::Bcc)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the CC header field
     pub fn get_cc(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::Cc)
+            .get(&RfcHeader::Cc)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Comments header fields
     pub fn get_comments(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::Comments)
+            .get(&RfcHeader::Comments)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Date header field
     pub fn get_date(&self) -> Option<&DateTime> {
         self.headers_rfc
-            .get(&HeaderName::Date)
+            .get(&RfcHeader::Date)
             .and_then(|header| header.as_datetime_ref())
     }
 
     /// Returns the From header field
     pub fn get_from(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::From)
+            .get(&RfcHeader::From)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all In-Reply-To header fields
     pub fn get_in_reply_to(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::InReplyTo)
+            .get(&RfcHeader::InReplyTo)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Keywords header fields
     pub fn get_keywords(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::Keywords)
+            .get(&RfcHeader::Keywords)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Archive header field
     pub fn get_list_archive(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ListArchive)
+            .get(&RfcHeader::ListArchive)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Help header field
     pub fn get_list_help(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ListHelp)
+            .get(&RfcHeader::ListHelp)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-ID header field
     pub fn get_list_id(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ListId)
+            .get(&RfcHeader::ListId)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Owner header field
     pub fn get_list_owner(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ListOwner)
+            .get(&RfcHeader::ListOwner)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Post header field
     pub fn get_list_post(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ListPost)
+            .get(&RfcHeader::ListPost)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Subscribe header field
     pub fn get_list_subscribe(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ListSubscribe)
+            .get(&RfcHeader::ListSubscribe)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Unsubscribe header field
     pub fn get_list_unsubscribe(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ListUnsubscribe)
+            .get(&RfcHeader::ListUnsubscribe)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Message-ID header field
     pub fn get_message_id(&self) -> Option<&str> {
         self.headers_rfc
-            .get(&HeaderName::MessageId)
+            .get(&RfcHeader::MessageId)
             .and_then(|header| header.as_text_ref())
     }
 
     /// Returns the MIME-Version header field
     pub fn get_mime_version(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::MimeVersion)
+            .get(&RfcHeader::MimeVersion)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Received header fields
     pub fn get_received(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::Received)
+            .get(&RfcHeader::Received)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all References header fields
     pub fn get_references(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::References)
+            .get(&RfcHeader::References)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Reply-To header field
     pub fn get_reply_to(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ReplyTo)
+            .get(&RfcHeader::ReplyTo)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Resent-BCC header field
     pub fn get_resent_bcc(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ResentBcc)
+            .get(&RfcHeader::ResentBcc)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Resent-CC header field
     pub fn get_resent_cc(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ResentTo)
+            .get(&RfcHeader::ResentTo)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Resent-Date header fields
     pub fn get_resent_date(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ResentDate)
+            .get(&RfcHeader::ResentDate)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Resent-From header field
     pub fn get_resent_from(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ResentFrom)
+            .get(&RfcHeader::ResentFrom)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Resent-Message-ID header fields
     pub fn get_resent_message_id(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ResentMessageId)
+            .get(&RfcHeader::ResentMessageId)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Sender header field
     pub fn get_resent_sender(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ResentSender)
+            .get(&RfcHeader::ResentSender)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Resent-To header field
     pub fn get_resent_to(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ResentTo)
+            .get(&RfcHeader::ResentTo)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Return-Path header fields
     pub fn get_return_path(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ReturnPath)
+            .get(&RfcHeader::ReturnPath)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Sender header field
     pub fn get_sender(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::Sender)
+            .get(&RfcHeader::Sender)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Subject header field
     pub fn get_subject(&self) -> Option<&str> {
         self.headers_rfc
-            .get(&HeaderName::Subject)
+            .get(&RfcHeader::Subject)
             .and_then(|header| header.as_text_ref())
     }
 
-    /// Returns the message thread name
+    /// Returns the message thread name or 'base subject' as defined in
+    /// [RFC 5957 - Internet Message Access Protocol - SORT and THREAD Extensions (Section 2.1)](https://datatracker.ietf.org/doc/html/rfc5256#section-2.1)
     pub fn get_thread_name(&self) -> Option<&str> {
         thread_name(self.get_subject()?).into()
     }
@@ -838,15 +987,40 @@ impl<'x> Message<'x> {
     /// Returns the To header field
     pub fn get_to(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::To)
+            .get(&RfcHeader::To)
             .unwrap_or(&HeaderValue::Empty)
     }
 
-    /// Returns a non-standard header field
-    pub fn get_other(&self, name: &'x str) -> &HeaderValue<'x> {
-        self.headers_other
-            .get(&Cow::from(name))
-            .unwrap_or(&HeaderValue::Empty)
+    /// Returns a preview of the message body
+    pub fn get_body_preview(&self, preview_len: usize) -> Option<Cow<'x, str>> {
+        if !self.text_body.is_empty() {
+            preview_text(self.get_text_body(0)?, preview_len).into()
+        } else if !self.html_body.is_empty() {
+            preview_html(self.get_html_body(0)?, preview_len).into()
+        } else {
+            None
+        }
+    }
+
+    pub fn _get_raw(&'x self, name: HeaderName) -> Option<Vec<Cow<'x, str>>> {
+        self.headers_raw
+            .get(&name)?
+            .iter()
+            .map(|offset| {
+                String::from_utf8_lossy(self.raw_message.get(offset.start..offset.end).unwrap())
+            })
+            .collect::<Vec<Cow<str>>>()
+            .into()
+    }
+
+    /// Returns in raw format a header field not defined in the conformed RFCs
+    pub fn get_other_header(&'x self, name: &str) -> Option<Vec<Cow<'x, str>>> {
+        self._get_raw(HeaderName::Other(name.into()))
+    }
+
+    /// Returns in raw format a header field defined in the conformed RFCs
+    pub fn get_rfc_header(&'x self, name: RfcHeader) -> Option<Vec<Cow<'x, str>>> {
+        self._get_raw(HeaderName::Rfc(name))
     }
 
     fn get_part(&self, list: &'x [MessagePartId], pos: usize) -> Option<&'x dyn BodyPart> {
@@ -962,43 +1136,43 @@ pub trait MimeHeaders<'x> {
 impl<'x> MimeHeaders<'x> for Message<'x> {
     fn get_content_description(&self) -> Option<&str> {
         self.headers_rfc
-            .get(&HeaderName::ContentDescription)
+            .get(&RfcHeader::ContentDescription)
             .and_then(|header| header.as_text_ref())
     }
 
     fn get_content_disposition(&self) -> Option<&ContentType> {
         self.headers_rfc
-            .get(&HeaderName::ContentDisposition)
+            .get(&RfcHeader::ContentDisposition)
             .and_then(|header| header.as_content_type_ref())
     }
 
     fn get_content_id(&self) -> Option<&str> {
         self.headers_rfc
-            .get(&HeaderName::ContentId)
+            .get(&RfcHeader::ContentId)
             .and_then(|header| header.as_text_ref())
     }
 
     fn get_content_transfer_encoding(&self) -> Option<&str> {
         self.headers_rfc
-            .get(&HeaderName::ContentTransferEncoding)
+            .get(&RfcHeader::ContentTransferEncoding)
             .and_then(|header| header.as_text_ref())
     }
 
     fn get_content_type(&self) -> Option<&ContentType> {
         self.headers_rfc
-            .get(&HeaderName::ContentType)
+            .get(&RfcHeader::ContentType)
             .and_then(|header| header.as_content_type_ref())
     }
 
     fn get_content_language(&self) -> &HeaderValue<'x> {
         self.headers_rfc
-            .get(&HeaderName::ContentLanguage)
+            .get(&RfcHeader::ContentLanguage)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     fn get_content_location(&self) -> Option<&str> {
         self.headers_rfc
-            .get(&HeaderName::ContentLocation)
+            .get(&RfcHeader::ContentLocation)
             .and_then(|header| header.as_text_ref())
     }
 }
@@ -1118,44 +1292,44 @@ impl<'x> fmt::Display for Part<'x, MessageAttachment<'x>> {
 
 impl<'x, T> MimeHeaders<'x> for Part<'x, T> {
     fn get_content_description(&self) -> Option<&str> {
-        self.headers
-            .get(&HeaderName::ContentDescription)
+        self.headers_rfc
+            .get(&RfcHeader::ContentDescription)
             .and_then(|header| header.as_text_ref())
     }
 
     fn get_content_disposition(&self) -> Option<&ContentType> {
-        self.headers
-            .get(&HeaderName::ContentDisposition)
+        self.headers_rfc
+            .get(&RfcHeader::ContentDisposition)
             .and_then(|header| header.as_content_type_ref())
     }
 
     fn get_content_id(&self) -> Option<&str> {
-        self.headers
-            .get(&HeaderName::ContentId)
+        self.headers_rfc
+            .get(&RfcHeader::ContentId)
             .and_then(|header| header.as_text_ref())
     }
 
     fn get_content_transfer_encoding(&self) -> Option<&str> {
-        self.headers
-            .get(&HeaderName::ContentTransferEncoding)
+        self.headers_rfc
+            .get(&RfcHeader::ContentTransferEncoding)
             .and_then(|header| header.as_text_ref())
     }
 
     fn get_content_type(&self) -> Option<&ContentType> {
-        self.headers
-            .get(&HeaderName::ContentType)
+        self.headers_rfc
+            .get(&RfcHeader::ContentType)
             .and_then(|header| header.as_content_type_ref())
     }
 
     fn get_content_language(&self) -> &HeaderValue<'x> {
-        self.headers
-            .get(&HeaderName::ContentLanguage)
+        self.headers_rfc
+            .get(&RfcHeader::ContentLanguage)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     fn get_content_location(&self) -> Option<&str> {
-        self.headers
-            .get(&HeaderName::ContentLocation)
+        self.headers_rfc
+            .get(&RfcHeader::ContentLocation)
             .and_then(|header| header.as_text_ref())
     }
 }
