@@ -11,19 +11,24 @@
 
 use std::{borrow::Cow, collections::hash_map::Entry};
 
-use crate::{HeaderName, HeaderOffset, HeaderOffsetName, HeaderValue, OtherHeaders, RfcHeaders};
+use crate::{HeaderName, HeaderOffset, HeaderValue, RawHeaders, RfcHeader, RfcHeaders};
 
 use super::{
     fields::{
-        address::parse_address, content_type::parse_content_type, date::parse_date, id::parse_id,
-        list::parse_comma_separared, raw::parse_raw, unstructured::parse_unstructured,
+        address::parse_address,
+        content_type::parse_content_type,
+        date::parse_date,
+        id::parse_id,
+        list::parse_comma_separared,
+        raw::{parse_and_ignore, parse_raw},
+        unstructured::parse_unstructured,
     },
     message::MessageStream,
 };
 
 #[derive(Debug, PartialEq)]
 pub enum HeaderParserResult<'x> {
-    Rfc(HeaderName),
+    Rfc(RfcHeader),
     Other(Cow<'x, str>),
     Lf,
     Eof,
@@ -31,8 +36,7 @@ pub enum HeaderParserResult<'x> {
 
 pub fn parse_headers<'x>(
     headers_rfc: &mut RfcHeaders<'x>,
-    mut headers_other: Option<&mut OtherHeaders<'x>>,
-    mut headers_offsets: Option<&mut Vec<HeaderOffset<'x>>>,
+    headers_raw: &mut RawHeaders<'x>,
     stream: &mut MessageStream<'x>,
 ) -> bool {
     loop {
@@ -41,72 +45,45 @@ pub fn parse_headers<'x>(
 
         match result {
             HeaderParserResult::Rfc(name) => {
-                let (is_many, parser) = HDR_PARSER[name as usize];
+                let (_, parser) = HDR_PARSER[name as usize];
 
                 let from_offset = stream.pos;
                 let value = parser(stream);
-                if let Some(headers_offsets) = headers_offsets.as_mut() {
-                    headers_offsets.push(HeaderOffset {
-                        name: HeaderOffsetName::Rfc(name),
+                headers_raw
+                    .entry(HeaderName::Rfc(name))
+                    .or_insert_with(Vec::new)
+                    .push(HeaderOffset {
                         start: from_offset,
                         end: stream.pos,
                     });
-                }
 
                 if !value.is_empty() {
-                    if is_many {
-                        match headers_rfc.entry(name) {
-                            Entry::Occupied(mut e) => {
-                                if let HeaderValue::Collection(col) = e.get_mut() {
-                                    col.push(value);
-                                } else {
-                                    let old_value = e.remove();
-                                    headers_rfc.insert(
-                                        name,
-                                        HeaderValue::Collection(vec![old_value, value]),
-                                    );
-                                }
-                            }
-                            Entry::Vacant(e) => {
-                                e.insert(value);
+                    match headers_rfc.entry(name) {
+                        Entry::Occupied(mut e) => {
+                            if let HeaderValue::Collection(col) = e.get_mut() {
+                                col.push(value);
+                            } else {
+                                let old_value = e.remove();
+                                headers_rfc
+                                    .insert(name, HeaderValue::Collection(vec![old_value, value]));
                             }
                         }
-                    } else {
-                        headers_rfc.insert(name, value);
+                        Entry::Vacant(e) => {
+                            e.insert(value);
+                        }
                     }
                 }
             }
             HeaderParserResult::Other(name) => {
-                if let Some(headers_other) = headers_other.as_mut() {
-                    let from_offset = stream.pos;
-                    let value = parse_raw(stream);
-                    if let Some(headers_offsets) = headers_offsets.as_mut() {
-                        headers_offsets.push(HeaderOffset {
-                            name: HeaderOffsetName::Other(name.clone()),
-                            start: from_offset,
-                            end: stream.pos,
-                        });
-                    }
-
-                    if !value.is_empty() {
-                        match headers_other.entry(name.clone()) {
-                            Entry::Occupied(mut e) => {
-                                if let HeaderValue::Collection(col) = e.get_mut() {
-                                    col.push(value);
-                                } else {
-                                    let old_value = e.remove();
-                                    headers_other.insert(
-                                        name,
-                                        HeaderValue::Collection(vec![old_value, value]),
-                                    );
-                                }
-                            }
-                            Entry::Vacant(e) => {
-                                e.insert(value);
-                            }
-                        }
-                    }
-                }
+                let from_offset = stream.pos;
+                parse_and_ignore(stream);
+                headers_raw
+                    .entry(HeaderName::Other(name))
+                    .or_insert_with(Vec::new)
+                    .push(HeaderOffset {
+                        start: from_offset,
+                        end: stream.pos,
+                    });
             }
             HeaderParserResult::Lf => return true,
             HeaderParserResult::Eof => return false,
@@ -173,27 +150,24 @@ pub fn parse_header_name(data: &[u8]) -> (usize, HeaderParserResult) {
     (bytes_read, HeaderParserResult::Eof)
 }
 
-impl From<HeaderName> for u8 {
-    fn from(name: HeaderName) -> Self {
+impl From<RfcHeader> for u8 {
+    fn from(name: RfcHeader) -> Self {
         name as u8
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::{parsers::header::parse_header_name, HeaderName};
+    use crate::{parsers::header::parse_header_name, RfcHeader};
 
     use super::HeaderParserResult;
 
     #[test]
     fn header_name_parse() {
         let inputs = [
-            ("From: ", HeaderParserResult::Rfc(HeaderName::From)),
-            ("receiVED: ", HeaderParserResult::Rfc(HeaderName::Received)),
-            (
-                " subject   : ",
-                HeaderParserResult::Rfc(HeaderName::Subject),
-            ),
+            ("From: ", HeaderParserResult::Rfc(RfcHeader::From)),
+            ("receiVED: ", HeaderParserResult::Rfc(RfcHeader::Received)),
+            (" subject   : ", HeaderParserResult::Rfc(RfcHeader::Subject)),
             (
                 "X-Custom-Field : ",
                 HeaderParserResult::Other("X-Custom-Field".into()),
@@ -205,7 +179,7 @@ mod tests {
             ),
             (
                 "MIME-version : ",
-                HeaderParserResult::Rfc(HeaderName::MimeVersion),
+                HeaderParserResult::Rfc(RfcHeader::MimeVersion),
             ),
         ];
 
@@ -231,7 +205,7 @@ static HDR_PARSER: &[(
     (false, parse_address),         // ReplyTo = 6,
     (false, parse_address),         // Sender = 7,
     (true, parse_unstructured),     // Comments = 8,
-    (false, parse_address),         // InReplyTo = 9,
+    (false, parse_id),              // InReplyTo = 9,
     (true, parse_comma_separared),  // Keywords = 10,
     (true, parse_raw),              // Received = 11,
     (false, parse_id),              // MessageId = 12,
@@ -276,76 +250,76 @@ static HDR_HASH: &[u8] = &[
     73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73,
 ];
 
-static HDR_MAP: &[HeaderName] = &[
-    HeaderName::Date,
-    HeaderName::Other,
-    HeaderName::Sender,
-    HeaderName::Other,
-    HeaderName::Received,
-    HeaderName::Other,
-    HeaderName::References,
-    HeaderName::Other,
-    HeaderName::Cc,
-    HeaderName::Comments,
-    HeaderName::ResentCc,
-    HeaderName::ContentId,
-    HeaderName::Other,
-    HeaderName::ResentMessageId,
-    HeaderName::ReplyTo,
-    HeaderName::ResentTo,
-    HeaderName::ResentBcc,
-    HeaderName::ContentLanguage,
-    HeaderName::Subject,
-    HeaderName::ResentSender,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::ResentDate,
-    HeaderName::To,
-    HeaderName::Bcc,
-    HeaderName::Other,
-    HeaderName::ContentTransferEncoding,
-    HeaderName::ReturnPath,
-    HeaderName::ListId,
-    HeaderName::Keywords,
-    HeaderName::ContentDescription,
-    HeaderName::ListOwner,
-    HeaderName::Other,
-    HeaderName::ContentType,
-    HeaderName::Other,
-    HeaderName::ListHelp,
-    HeaderName::MessageId,
-    HeaderName::ContentLocation,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::ListSubscribe,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::ListPost,
-    HeaderName::Other,
-    HeaderName::ResentFrom,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::ContentDisposition,
-    HeaderName::Other,
-    HeaderName::InReplyTo,
-    HeaderName::ListArchive,
-    HeaderName::Other,
-    HeaderName::From,
-    HeaderName::Other,
-    HeaderName::ListUnsubscribe,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::Other,
-    HeaderName::MimeVersion,
+static HDR_MAP: &[RfcHeader] = &[
+    RfcHeader::Date,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::Sender,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::Received,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::References,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::Cc,
+    RfcHeader::Comments,
+    RfcHeader::ResentCc,
+    RfcHeader::ContentId,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ResentMessageId,
+    RfcHeader::ReplyTo,
+    RfcHeader::ResentTo,
+    RfcHeader::ResentBcc,
+    RfcHeader::ContentLanguage,
+    RfcHeader::Subject,
+    RfcHeader::ResentSender,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ResentDate,
+    RfcHeader::To,
+    RfcHeader::Bcc,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ContentTransferEncoding,
+    RfcHeader::ReturnPath,
+    RfcHeader::ListId,
+    RfcHeader::Keywords,
+    RfcHeader::ContentDescription,
+    RfcHeader::ListOwner,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ContentType,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ListHelp,
+    RfcHeader::MessageId,
+    RfcHeader::ContentLocation,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ListSubscribe,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ListPost,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ResentFrom,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ContentDisposition,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::InReplyTo,
+    RfcHeader::ListArchive,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::From,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::ListUnsubscribe,
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion, // Invalid
+    RfcHeader::MimeVersion,
 ];
 
 static HDR_NAMES: &[&[u8]] = &[
