@@ -17,7 +17,7 @@ use crate::{
         quoted_printable::decode_quoted_printable, DecodeFnc, DecodeResult,
     },
     ContentType, HeaderName, HeaderValue, Message, MessageAttachment, MessagePart, MessagePartId,
-    MessageStructure, Part, RfcHeaders,
+    MessageStructure, MultiPart, Part, RawHeaders, RfcHeader, RfcHeaders,
 };
 
 use super::{
@@ -122,7 +122,7 @@ fn add_missing_type<'x>(
 ) {
     if headers.is_empty() {
         headers.insert(
-            HeaderName::ContentType,
+            RfcHeader::ContentType,
             HeaderValue::ContentType(ContentType {
                 c_type,
                 c_subtype: Some(c_subtype),
@@ -194,7 +194,7 @@ impl<'x> Message<'x> {
 
     /// Returns `false` if at least one header field was successfully parsed.
     pub fn is_empty(&self) -> bool {
-        self.headers_rfc.is_empty() && self.headers_other.is_empty()
+        self.headers_rfc.is_empty() && self.headers_raw.is_empty()
     }
 
     /// Parses a byte slice containing the RFC5322 raw message and returns a
@@ -213,6 +213,7 @@ impl<'x> Message<'x> {
         let mut state_stack = Vec::new();
 
         let mut mime_part_header = RfcHeaders::new();
+        let mut mime_part_header_raw = RawHeaders::new();
 
         'outer: loop {
             // Parse headers
@@ -220,8 +221,7 @@ impl<'x> Message<'x> {
                 message.offset_header = stream.pos;
                 if !parse_headers(
                     &mut message.headers_rfc,
-                    Some(&mut message.headers_other),
-                    Some(&mut message.headers_offsets),
+                    &mut message.headers_raw,
                     &mut stream,
                 ) {
                     break;
@@ -229,7 +229,11 @@ impl<'x> Message<'x> {
                 message.offset_body = seek_crlf_end(&stream, stream.pos);
                 (true, &mut message.headers_rfc)
             } else {
-                if !parse_headers(&mut mime_part_header, None, None, &mut stream) {
+                if !parse_headers(
+                    &mut mime_part_header,
+                    &mut mime_part_header_raw,
+                    &mut stream,
+                ) {
                     break;
                 }
                 (false, &mut mime_part_header)
@@ -238,7 +242,7 @@ impl<'x> Message<'x> {
             state.parts += 1;
 
             let content_type = header
-                .get(&HeaderName::ContentType)
+                .get(&RfcHeader::ContentType)
                 .and_then(|c| c.as_content_type_ref());
 
             let (is_multipart, mut is_inline, mut is_text, mut mime_type) =
@@ -269,8 +273,9 @@ impl<'x> Message<'x> {
                         };
                         if !is_message {
                             add_missing_type(&mut mime_part_header, "text".into(), "plain".into());
-                            message.parts.push(MessagePart::Multipart(std::mem::take(
-                                &mut mime_part_header,
+                            message.parts.push(MessagePart::Multipart(MultiPart::new(
+                                std::mem::take(&mut mime_part_header),
+                                std::mem::take(&mut mime_part_header_raw),
                             )));
                         }
                         state_stack.push(state);
@@ -287,7 +292,7 @@ impl<'x> Message<'x> {
             skip_crlf(&mut stream);
 
             let (is_binary, decode_fnc): (bool, DecodeFnc) = match header
-                .get(&HeaderName::ContentTransferEncoding)
+                .get(&RfcHeader::ContentTransferEncoding)
             {
                 Some(HeaderValue::Text(encoding)) if encoding.eq_ignore_ascii_case("base64") => {
                     (false, decode_base64)
@@ -313,7 +318,11 @@ impl<'x> Message<'x> {
                     ..Default::default()
                 };
                 add_missing_type(&mut mime_part_header, "message".into(), "rfc822".into());
-                message_stack.push((message, std::mem::take(&mut mime_part_header)));
+                message_stack.push((
+                    message,
+                    std::mem::take(&mut mime_part_header),
+                    std::mem::take(&mut mime_part_header_raw),
+                ));
                 state_stack.push(state);
                 message = Message::new();
                 state = new_state;
@@ -331,7 +340,8 @@ impl<'x> Message<'x> {
             );
 
             // Attempt to recover contents of an invalid message
-            if bytes_read == 0 {
+            let is_encoding_problem = bytes_read == 0;
+            if is_encoding_problem {
                 if stream.pos >= stream.data.len() || (is_binary && state.mime_boundary.is_none()) {
                     state.structure.pop();
                     break;
@@ -382,7 +392,7 @@ impl<'x> Message<'x> {
             if mime_type != MimeType::Message {
                 let is_inline = is_inline
                     && header
-                        .get(&HeaderName::ContentDisposition)
+                        .get(&RfcHeader::ContentDisposition)
                         .map_or_else(|| true, |d| !d.get_content_type().is_attachment())
                     && (state.parts == 1
                         || (state.mime_type != MimeType::MultipartRelated
@@ -416,11 +426,39 @@ impl<'x> Message<'x> {
 
                 if is_text {
                     let is_html = mime_type == MimeType::TextHtml;
-                    let text_part = Part {
+                    let mut text_part = Part {
                         body: result_to_string(bytes, stream.data, content_type),
-                        headers: std::mem::take(&mut mime_part_header),
+                        headers_rfc: std::mem::take(&mut mime_part_header),
+                        headers_raw: std::mem::take(&mut mime_part_header_raw),
+                        is_encoding_problem,
                     };
 
+                    // If there is a single part in the message, move MIME headers
+                    // to the part
+                    if is_message {
+                        for header_name in [
+                            RfcHeader::ContentType,
+                            RfcHeader::ContentDisposition,
+                            RfcHeader::ContentId,
+                            RfcHeader::ContentLanguage,
+                            RfcHeader::ContentLocation,
+                            RfcHeader::ContentTransferEncoding,
+                            RfcHeader::ContentDescription,
+                        ] {
+                            if let Some(value) = message.headers_rfc.remove(&header_name) {
+                                text_part.headers_rfc.insert(header_name, value);
+                            }
+                            if let Some(value) =
+                                message.headers_raw.get(&HeaderName::Rfc(header_name))
+                            {
+                                text_part
+                                    .headers_raw
+                                    .insert(HeaderName::Rfc(header_name), value.to_vec());
+                            }
+                        }
+                    }
+                    add_missing_type(&mut text_part.headers_rfc, "text".into(), "plain".into());
+
                     if add_to_html && !is_html {
                         message.html_body.push(message.parts.len());
                     } else if add_to_text && is_html {
@@ -450,11 +488,38 @@ impl<'x> Message<'x> {
 
                     message.attachments.push(message.parts.len());
 
-                    let binary_part = Part::new(
+                    let mut binary_part = Part::new(
                         std::mem::take(&mut mime_part_header),
+                        std::mem::take(&mut mime_part_header_raw),
                         result_to_bytes(bytes, stream.data),
+                        is_encoding_problem,
                     );
 
+                    // If there is a single part in the message, move MIME headers
+                    // to the part
+                    if is_message {
+                        for header_name in [
+                            RfcHeader::ContentType,
+                            RfcHeader::ContentDisposition,
+                            RfcHeader::ContentId,
+                            RfcHeader::ContentLanguage,
+                            RfcHeader::ContentLocation,
+                            RfcHeader::ContentTransferEncoding,
+                            RfcHeader::ContentDescription,
+                        ] {
+                            if let Some(value) = message.headers_rfc.remove(&header_name) {
+                                binary_part.headers_rfc.insert(header_name, value);
+                            }
+                            if let Some(value) =
+                                message.headers_raw.get(&HeaderName::Rfc(header_name))
+                            {
+                                binary_part
+                                    .headers_raw
+                                    .insert(HeaderName::Rfc(header_name), value.to_vec());
+                            }
+                        }
+                    }
+
                     message.parts.push(if !is_inline {
                         MessagePart::Binary(binary_part)
                     } else {
@@ -466,7 +531,9 @@ impl<'x> Message<'x> {
                 message.attachments.push(message.parts.len());
                 message.parts.push(MessagePart::Message(Part::new(
                     std::mem::take(&mut mime_part_header),
+                    std::mem::take(&mut mime_part_header_raw),
                     MessageAttachment::Raw(result_to_bytes(bytes, stream.data)),
+                    is_encoding_problem,
                 )));
             }
 
@@ -477,8 +544,10 @@ impl<'x> Message<'x> {
                 'inner: loop {
                     if let MimeType::Message = state.mime_type {
                         // Finished processing nested message, restore parent message from stack
-                        if let (Some((mut prev_message, headers)), Some(mut prev_state)) =
-                            (message_stack.pop(), state_stack.pop())
+                        if let (
+                            Some((mut prev_message, headers, headers_raw)),
+                            Some(mut prev_state),
+                        ) = (message_stack.pop(), state_stack.pop())
                         {
                             message.structure = state.get_structure();
                             message.offset_end = seek_crlf_end(&stream, last_part_offset);
@@ -487,7 +556,9 @@ impl<'x> Message<'x> {
                             prev_message.attachments.push(prev_message.parts.len());
                             prev_message.parts.push(MessagePart::Message(Part::new(
                                 headers,
+                                headers_raw,
                                 MessageAttachment::Parsed(Box::new(message)),
+                                false,
                             )));
                             message = prev_message;
                             prev_state.mime_boundary = state.mime_boundary;
@@ -551,7 +622,7 @@ impl<'x> Message<'x> {
             }
         }
 
-        while let (Some((mut prev_message, headers)), Some(prev_state)) =
+        while let (Some((mut prev_message, headers, headers_raw)), Some(prev_state)) =
             (message_stack.pop(), state_stack.pop())
         {
             message.offset_end = stream.pos;
@@ -561,7 +632,9 @@ impl<'x> Message<'x> {
                 prev_message.attachments.push(prev_message.parts.len());
                 prev_message.parts.push(MessagePart::Message(Part::new(
                     headers,
+                    headers_raw,
                     MessageAttachment::Parsed(Box::new(message)),
+                    false,
                 )));
             }
             message = prev_message;
@@ -587,7 +660,7 @@ mod tests {
 
     use crate::parsers::message::Message;
 
-    const SEPARATOR: &[u8] = "\n---- EXPECTED STRUCTURE ----\n".as_bytes();
+    const SEPARATOR: &[u8] = "\n---- EXPECTED STRUCTURE ----".as_bytes();
 
     #[test]
     fn parse_full_messages() {
@@ -644,8 +717,7 @@ mod tests {
         }
     }
 
-    /*/
-    #[test]
+    /*#[test]
     fn message_to_yaml() {
         let mut file_name = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
         file_name.push("tests");
